"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.AnyJsonSchema = void 0;
exports.ArrayJsonSchema = ArrayJsonSchema;
exports.BooleanJsonSchema = exports.BigIntJsonSchema = void 0;
exports.EqJsonSchema = EqJsonSchema;
Object.defineProperty(exports, "Functor", {
  enumerable: true,
  get: function () {
    return _functor.Functor;
  }
});
exports.IntegerJsonSchema = exports.InlineJsonSchema = void 0;
exports.IntersectJsonSchema = IntersectJsonSchema;
exports.NumberJsonSchema = exports.NullJsonSchema = exports.NeverJsonSchema = void 0;
exports.ObjectJsonSchema = ObjectJsonSchema;
exports.OptionalJsonSchema = OptionalJsonSchema;
Object.defineProperty(exports, "RAW", {
  enumerable: true,
  get: function () {
    return Spec.RAW;
  }
});
exports.RecordJsonSchema = RecordJsonSchema;
exports.SymbolJsonSchema = exports.StringJsonSchema = void 0;
exports.TupleJsonSchema = TupleJsonSchema;
exports.UndefinedJsonSchema = void 0;
exports.UnionJsonSchema = UnionJsonSchema;
exports.VoidJsonSchema = exports.UnknownJsonSchema = void 0;
Object.defineProperty(exports, "fold", {
  enumerable: true,
  get: function () {
    return _functor.fold;
  }
});
Object.defineProperty(exports, "is", {
  enumerable: true,
  get: function () {
    return Spec.is;
  }
});
Object.defineProperty(exports, "isRequired", {
  enumerable: true,
  get: function () {
    return _properties.isRequired;
  }
});
Object.defineProperty(exports, "minItems", {
  enumerable: true,
  get: function () {
    return _items.minItems;
  }
});
Object.defineProperty(exports, "property", {
  enumerable: true,
  get: function () {
    return _properties.property;
  }
});
Object.defineProperty(exports, "unfold", {
  enumerable: true,
  get: function () {
    return _functor.unfold;
  }
});
Object.defineProperty(exports, "wrapOptional", {
  enumerable: true,
  get: function () {
    return _properties.wrapOptional;
  }
});
var _registry = /*#__PURE__*/require("@traversable/registry");
var _schema = /*#__PURE__*/require("@traversable/schema");
var _items = /*#__PURE__*/require("./items.js");
var _properties = /*#__PURE__*/require("./properties.js");
var Spec = /*#__PURE__*/_interopRequireWildcard(/*#__PURE__*/require("./specification.js"));
var _functor = /*#__PURE__*/require("./functor.js");
function _getRequireWildcardCache(e) {
  if ("function" != typeof WeakMap) return null;
  var r = new WeakMap(),
    t = new WeakMap();
  return (_getRequireWildcardCache = function (e) {
    return e ? t : r;
  })(e);
}
function _interopRequireWildcard(e, r) {
  if (!r && e && e.__esModule) return e;
  if (null === e || "object" != typeof e && "function" != typeof e) return {
    default: e
  };
  var t = _getRequireWildcardCache(r);
  if (t && t.has(e)) return t.get(e);
  var n = {
      __proto__: null
    },
    a = Object.defineProperty && Object.getOwnPropertyDescriptor;
  for (var u in e) if ("default" !== u && {}.hasOwnProperty.call(e, u)) {
    var i = a ? Object.getOwnPropertyDescriptor(e, u) : null;
    i && (i.get || i.set) ? Object.defineProperty(n, u, i) : n[u] = e[u];
  }
  return n.default = e, t && t.set(e, n), n;
}
/** @internal */
const Object_keys = globalThis.Object.keys;
/** @internal */
const Object_assign = globalThis.Object.assign;
const hasSchema = /*#__PURE__*/(0, _registry.has)('jsonSchema');
const getSchema = u => hasSchema(u) ? u.jsonSchema : u;
function applyTupleOptionality(xs, {
  min,
  max
}) {
  return min === max ? xs.map(getSchema) : [...xs.slice(0, min).map(getSchema), ...xs.slice(min).map(getSchema)];
}
const NeverJsonSchema = exports.NeverJsonSchema = {
  get jsonSchema() {
    return void 0;
  }
};
const VoidJsonSchema = exports.VoidJsonSchema = {
  get jsonSchema() {
    return void 0;
  }
};
const UndefinedJsonSchema = exports.UndefinedJsonSchema = {
  get jsonSchema() {
    return void 0;
  }
};
const SymbolJsonSchema = exports.SymbolJsonSchema = {
  get jsonSchema() {
    return void 0;
  }
};
const BigIntJsonSchema = exports.BigIntJsonSchema = {
  get jsonSchema() {
    return void 0;
  }
};
const InlineJsonSchema = predicate => Object_assign(_schema.t.inline(predicate), {
  jsonSchema: void 0
});
/* * * * * * * * *
 *                *
 *   data types   *
 *                *
  * * * * * * * * */
exports.InlineJsonSchema = InlineJsonSchema;
const AnyJsonSchema = exports.AnyJsonSchema = {
  get jsonSchema() {
    return Spec.RAW.any;
  }
};
const UnknownJsonSchema = exports.UnknownJsonSchema = {
  get jsonSchema() {
    return Spec.RAW.any;
  }
};
const NullJsonSchema = exports.NullJsonSchema = {
  get jsonSchema() {
    return Spec.RAW.null;
  }
};
const BooleanJsonSchema = exports.BooleanJsonSchema = {
  get jsonSchema() {
    return Spec.RAW.boolean;
  }
};
const IntegerJsonSchema = exports.IntegerJsonSchema = {
  get jsonSchema() {
    return Spec.RAW.integer;
  }
};
const NumberJsonSchema = exports.NumberJsonSchema = {
  get jsonSchema() {
    return Spec.RAW.number;
  }
};
const StringJsonSchema = exports.StringJsonSchema = {
  get jsonSchema() {
    return Spec.RAW.string;
  }
};
function EqJsonSchema(value) {
  return {
    get jsonSchema() {
      return {
        const: value
      };
    }
  };
}
function OptionalJsonSchema(x) {
  return {
    get jsonSchema() {
      return {
        ...(hasSchema(x) && x.jsonSchema),
        [_registry.symbol.optional]: (0, _properties.wrapOptional)(x)
      };
    }
  };
}
function ArrayJsonSchema(x) {
  return {
    get jsonSchema() {
      return {
        type: 'array',
        items: getSchema(x)
      };
    }
  };
}
function RecordJsonSchema(x) {
  return {
    get jsonSchema() {
      return {
        type: 'object',
        additionalProperties: getSchema(x)
      };
    }
  };
}
function UnionJsonSchema(xs) {
  return {
    get jsonSchema() {
      return {
        anyOf: xs.map(getSchema)
      };
    }
  };
}
function IntersectJsonSchema(xs) {
  return {
    get jsonSchema() {
      return {
        allOf: xs.map(getSchema)
      };
    }
  };
}
function TupleJsonSchema(xs) {
  const min = (0, _items.minItems)(xs);
  const max = xs.length;
  return {
    get jsonSchema() {
      return {
        type: 'array',
        additionalItems: false,
        items: applyTupleOptionality(xs, {
          min,
          max
        }),
        minItems: min,
        maxItems: max
      };
    }
  };
}
function ObjectJsonSchema(xs) {
  const required = Object_keys(xs).filter((0, _properties.isRequired)(xs));
  return {
    get jsonSchema() {
      return {
        type: 'object',
        required,
        properties: _registry.fn.map(xs, (0, _properties.property)(required))
      };
    }
  };
}
//# sourceMappingURL=jsonSchema.js.map