"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.toJsonSchema = exports.fromJsonSchema = exports.Recursive = void 0;
var _registry = /*#__PURE__*/require("@traversable/registry");
var _schema = /*#__PURE__*/require("@traversable/schema");
var JsonSchema = /*#__PURE__*/_interopRequireWildcard(/*#__PURE__*/require("./jsonSchema.js"));
function _getRequireWildcardCache(e) {
  if ("function" != typeof WeakMap) return null;
  var r = new WeakMap(),
    t = new WeakMap();
  return (_getRequireWildcardCache = function (e) {
    return e ? t : r;
  })(e);
}
function _interopRequireWildcard(e, r) {
  if (!r && e && e.__esModule) return e;
  if (null === e || "object" != typeof e && "function" != typeof e) return {
    default: e
  };
  var t = _getRequireWildcardCache(r);
  if (t && t.has(e)) return t.get(e);
  var n = {
      __proto__: null
    },
    a = Object.defineProperty && Object.getOwnPropertyDescriptor;
  for (var u in e) if ("default" !== u && {}.hasOwnProperty.call(e, u)) {
    var i = a ? Object.getOwnPropertyDescriptor(e, u) : null;
    i && (i.get || i.set) ? Object.defineProperty(n, u, i) : n[u] = e[u];
  }
  return n.default = e, t && t.set(e, n), n;
}
/** @internal */
const phantom = () => void 0;
/** @internal */
function handleOptionality(x) {
  let {
    [_registry.symbol.optional]: ix,
    ...y
  } = x;
  y = Recursive.fromJsonSchema(y);
  if (typeof ix !== 'number') return Recursive.fromJsonSchema(y);else while (ix-- > 0) y = _schema.t.optional(y);
  return y;
}
/** @internal */
function recoverTupleOptionality(xs, {
  min,
  max
}) {
  if (min !== max) {
    const firstOptional = min;
    const req = xs.slice(0, firstOptional);
    const opt = xs.slice(firstOptional).map(_schema.t.optional.def);
    return [...req, ...opt];
  } else return xs;
}
var Recursive;
(function (Recursive) {
  Recursive.toJsonSchema = x => {
    switch (true) {
      default:
        return _registry.fn.exhaustive(x);
      case x.tag === _registry.URI.symbol:
        return phantom();
      case x.tag === _registry.URI.bigint:
        return phantom();
      case x.tag === _registry.URI.never:
        return phantom();
      case x.tag === _registry.URI.undefined:
        return phantom();
      case x.tag === _registry.URI.void:
        return phantom();
      case x.tag === _registry.URI.any:
        return JsonSchema.RAW.any;
      case x.tag === _registry.URI.unknown:
        return JsonSchema.RAW.any;
      case x.tag === _registry.URI.null:
        return JsonSchema.RAW.null;
      case x.tag === _registry.URI.boolean:
        return JsonSchema.RAW.boolean;
      case x.tag === _registry.URI.integer:
        return JsonSchema.RAW.integer;
      case x.tag === _registry.URI.number:
        return JsonSchema.RAW.number;
      case x.tag === _registry.URI.string:
        return JsonSchema.RAW.string;
      case x.tag === _registry.URI.optional:
        return JsonSchema.OptionalJsonSchema({
          jsonSchema: x.def
        }).jsonSchema;
      case x.tag === _registry.URI.eq:
        return JsonSchema.EqJsonSchema(x.def).jsonSchema;
      case x.tag === _registry.URI.array:
        return JsonSchema.ArrayJsonSchema({
          jsonSchema: x.def
        }).jsonSchema;
      case x.tag === _registry.URI.record:
        return JsonSchema.RecordJsonSchema({
          ...x,
          jsonSchema: x.def
        }).jsonSchema;
      case x.tag === _registry.URI.union:
        return JsonSchema.UnionJsonSchema(_registry.fn.map(x.def, v => ({
          jsonSchema: v
        }))).jsonSchema;
      case x.tag === _registry.URI.intersect:
        return JsonSchema.IntersectJsonSchema(_registry.fn.map(x.def, v => ({
          jsonSchema: v
        }))).jsonSchema;
      case x.tag === _registry.URI.tuple:
        return JsonSchema.TupleJsonSchema(_registry.fn.map(x.def, v => ({
          jsonSchema: v
        }))).jsonSchema;
      case x.tag === _registry.URI.object:
        return JsonSchema.ObjectJsonSchema(_registry.fn.map(x.def, v => ({
          jsonSchema: v
        }))).jsonSchema;
    }
  };
  Recursive.fromJsonSchema = x => {
    switch (true) {
      default:
        return _registry.fn.exhaustive(x);
      case _registry.symbol.optional in x:
        return handleOptionality(x);
      case JsonSchema.is.null(x):
        return _schema.t.null;
      case JsonSchema.is.boolean(x):
        return _schema.t.boolean;
      case JsonSchema.is.integer(x):
        return _schema.t.integer;
      case JsonSchema.is.number(x):
        return _schema.t.number;
      case JsonSchema.is.string(x):
        return _schema.t.string;
      case JsonSchema.is.any(x):
        return _schema.t.unknown;
      case JsonSchema.is.record(x):
        return _schema.t.record(x.additionalProperties);
      case JsonSchema.is.union(x):
        return _schema.t.union.def(x.anyOf);
      case JsonSchema.is.enum(x):
        return _schema.t.union.def(x.enum.map(_ => _schema.t.eq(_)));
      case JsonSchema.is.const(x):
        return _schema.t.eq.def(x.const);
      case JsonSchema.is.intersect(x):
        return _schema.t.intersect.def(x.allOf);
      case JsonSchema.is.tuple(x):
        return _schema.t.tuple.def(recoverTupleOptionality(x.items, {
          min: x.minItems,
          max: x.maxItems
        }));
      case JsonSchema.is.array(x):
        return _schema.t.array.def(x.items);
      case JsonSchema.is.object(x):
        return _schema.t.object(_registry.fn.map(x.properties, (v, k) => x.required.includes(`${k}`) ? v : _schema.t.optional(v)));
    }
  };
})(Recursive || (exports.Recursive = Recursive = {}));
/**
 * ## {@link fromJsonSchema `fromJsonSchema`}
 *
 * Recursively convert a JSON schema into a traversable schema.
 *
 * __Roundtrip property:__
 *
 * Thoroughly tested using randomly generated schemas to be invertible when paired
 * with {@link toJsonSchema} without any loss of information.
 */
const fromJsonSchema = exports.fromJsonSchema = /*#__PURE__*/JsonSchema.fold(Recursive.fromJsonSchema);
/**
 * ## {@link toJsonSchema `toJsonSchema`}
 *
 * Recursively convert a traversable schema into its JSON schema representation.
 *
 * __Roundtrip property:__
 *
 * Thoroughly tested using randomly generated schemas to be invertible with
 * {@link fromJsonSchema `fromJsonSchema`} without any loss of information.
 *
 * __Note:__ If you find yourself reaching for {@link toJsonSchema `toJsonSchema`},
 * keep in mind that by installing the `@traversable/schema-to-json-schema` package,
 * all of the built-in schemas from the `@traversable/schema` package come
 * equipped with a `.toJsonSchema` getter method.
 *
 * Prefer using the `.toJsonSchema` method if possible -- not only is it
 * easier to use, but it's also more performant, since the implementation
 * is not recursive.
 */
const toJsonSchema = exports.toJsonSchema = /*#__PURE__*/_schema.t.fold(Recursive.toJsonSchema);
//# sourceMappingURL=recursive.js.map