import { fn, has, symbol } from '@traversable/registry';
import { t } from '@traversable/schema';
import { minItems } from './items.js';
import { isRequired, property, wrapOptional } from './properties.js';
import * as Spec from './specification.js';
export { is, RAW } from './specification.js';
export { Functor, fold, unfold } from './functor.js';
export { isRequired, property, wrapOptional } from './properties.js';
export { minItems } from './items.js';
export { /* schemas that don't have a corresponding JSON Schema representation */
SymbolJsonSchema, VoidJsonSchema, UndefinedJsonSchema, BigIntJsonSchema, NeverJsonSchema, InlineJsonSchema, /* data types */
UnknownJsonSchema, AnyJsonSchema, NullJsonSchema, BooleanJsonSchema, IntegerJsonSchema, NumberJsonSchema, StringJsonSchema, EqJsonSchema, OptionalJsonSchema, ArrayJsonSchema, RecordJsonSchema, TupleJsonSchema, UnionJsonSchema, IntersectJsonSchema, ObjectJsonSchema };
/** @internal */
const Object_keys = globalThis.Object.keys;
/** @internal */
const Object_assign = globalThis.Object.assign;
const hasSchema = /*#__PURE__*/has('jsonSchema');
const getSchema = u => hasSchema(u) ? u.jsonSchema : u;
function applyTupleOptionality(xs, {
  min,
  max
}) {
  return min === max ? xs.map(getSchema) : [...xs.slice(0, min).map(getSchema), ...xs.slice(min).map(getSchema)];
}
const NeverJsonSchema = {
  get jsonSchema() {
    return void 0;
  }
};
const VoidJsonSchema = {
  get jsonSchema() {
    return void 0;
  }
};
const UndefinedJsonSchema = {
  get jsonSchema() {
    return void 0;
  }
};
const SymbolJsonSchema = {
  get jsonSchema() {
    return void 0;
  }
};
const BigIntJsonSchema = {
  get jsonSchema() {
    return void 0;
  }
};
const InlineJsonSchema = predicate => Object_assign(t.inline(predicate), {
  jsonSchema: void 0
});
/* * * * * * * * *
 *                *
 *   data types   *
 *                *
  * * * * * * * * */
const AnyJsonSchema = {
  get jsonSchema() {
    return Spec.RAW.any;
  }
};
const UnknownJsonSchema = {
  get jsonSchema() {
    return Spec.RAW.any;
  }
};
const NullJsonSchema = {
  get jsonSchema() {
    return Spec.RAW.null;
  }
};
const BooleanJsonSchema = {
  get jsonSchema() {
    return Spec.RAW.boolean;
  }
};
const IntegerJsonSchema = {
  get jsonSchema() {
    return Spec.RAW.integer;
  }
};
const NumberJsonSchema = {
  get jsonSchema() {
    return Spec.RAW.number;
  }
};
const StringJsonSchema = {
  get jsonSchema() {
    return Spec.RAW.string;
  }
};
function EqJsonSchema(value) {
  return {
    get jsonSchema() {
      return {
        const: value
      };
    }
  };
}
function OptionalJsonSchema(x) {
  return {
    get jsonSchema() {
      return {
        ...(hasSchema(x) && x.jsonSchema),
        [symbol.optional]: wrapOptional(x)
      };
    }
  };
}
function ArrayJsonSchema(x) {
  return {
    get jsonSchema() {
      return {
        type: 'array',
        items: getSchema(x)
      };
    }
  };
}
function RecordJsonSchema(x) {
  return {
    get jsonSchema() {
      return {
        type: 'object',
        additionalProperties: getSchema(x)
      };
    }
  };
}
function UnionJsonSchema(xs) {
  return {
    get jsonSchema() {
      return {
        anyOf: xs.map(getSchema)
      };
    }
  };
}
function IntersectJsonSchema(xs) {
  return {
    get jsonSchema() {
      return {
        allOf: xs.map(getSchema)
      };
    }
  };
}
function TupleJsonSchema(xs) {
  const min = minItems(xs);
  const max = xs.length;
  return {
    get jsonSchema() {
      return {
        type: 'array',
        additionalItems: false,
        items: applyTupleOptionality(xs, {
          min,
          max
        }),
        minItems: min,
        maxItems: max
      };
    }
  };
}
function ObjectJsonSchema(xs) {
  const required = Object_keys(xs).filter(isRequired(xs));
  return {
    get jsonSchema() {
      return {
        type: 'object',
        required,
        properties: fn.map(xs, property(required))
      };
    }
  };
}
//# sourceMappingURL=jsonSchema.js.map